"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fhirResourceBundle = void 0;

var _object = require("../utils/object");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const RECEIVED = 'FHIR_RESOURCE_RECEIVED';
const LOADING = 'FHIR_RESOURCE_LOADING';
const DELETING = 'FHIR_RESOURCE_DELETING';
const DELETED = 'FHIR_RESOURCE_DELETED';
const UPDATING = 'FHIR_RESOURCE_UPDATING';

const createFhirResourceSelector = resourceName => {
  let lastResult;
  let lastIndex;
  let lastActiveOperations;
  return state => {
    const {
      byId,
      indexes,
      activeOperations
    } = state.fhirResourceCache;
    const resourceIndex = indexes[resourceName];

    if (resourceIndex === lastIndex && activeOperations === lastActiveOperations) {
      return lastResult;
    }

    const result = {};

    for (const key in byId) {
      const resource = byId[key];

      if (resource.resourceType === resourceName) {
        const activeOperation = activeOperations[key];

        if (activeOperation) {
          result[key] = _objectSpread(_objectSpread({}, resource), {}, {
            activeOperation
          });
        } else {
          result[key] = resource;
        }
      }
    }

    lastIndex = resourceIndex;
    lastResult = result;
    lastActiveOperations = activeOperations;
    return result;
  };
};

const fhirResourceBundle = {
  name: 'fhirResourceCache',
  getReducer: () => {
    const initial = {
      // we do these to enable better memoization
      // of resources
      indexes: {// Patient: {
        //   [id]: true
        // },
        // Location: {},
        // Appointment: {},
        // Practitioner: {}
      },
      activeOperations: {// [id]: 'delete', 'update', 'load'
      },
      byId: {// [id]: FHIR Resource
      }
    };
    return (state = initial, {
      type,
      payload
    }) => {
      if (type === RECEIVED) {
        const updatedIndexes = _objectSpread({}, state.indexes);

        const resourceUpdates = {};

        for (const key in payload) {
          const resource = payload[key];
          const {
            resourceType,
            id
          } = resource;
          updatedIndexes[resourceType] = _objectSpread(_objectSpread({}, state.indexes[resourceType]), {}, {
            [id]: true
          });
          resourceUpdates[id] = resource;
        }

        return {
          activeOperations: state.activeOperations,
          indexes: updatedIndexes,
          byId: _objectSpread(_objectSpread({}, state.byId), resourceUpdates)
        };
      }

      if (type === LOADING) {
        return _objectSpread(_objectSpread({}, state), {}, {
          activeOperations: _objectSpread(_objectSpread({}, state.activeOperations), {}, {
            [payload]: 'load'
          })
        });
      }

      if (type === DELETING) {
        return _objectSpread(_objectSpread({}, state), {}, {
          activeOperations: _objectSpread(_objectSpread({}, state.activeOperations), {}, {
            [payload]: 'delete'
          })
        });
      }

      if (type === DELETED) {
        return _objectSpread(_objectSpread({}, state), {}, {
          activeOperations: (0, _object.omit)(state.activeOperations, [payload]),
          byId: (0, _object.omit)(state.byId, [payload])
        });
      }

      if (type === UPDATING) {
        return _objectSpread(_objectSpread({}, state), {}, {
          activeOperations: _objectSpread(_objectSpread({}, state.activeOperations), {}, {
            [payload]: 'update'
          })
        });
      }

      return state;
    };
  },
  selectPatientsRaw: createFhirResourceSelector('Patient'),
  selectLocationsRaw: createFhirResourceSelector('Location'),
  selectPractitionersRaw: createFhirResourceSelector('Practitioner'),
  selectAppointmentsRaw: createFhirResourceSelector('Appointment'),
  selectCompositionsRaw: createFhirResourceSelector('Composition'),
  selectOrganizationsRaw: createFhirResourceSelector('Organization'),
  selectFhirResourcesById: state => state.fhirResourceCache.byId,
  doDeleteFhirResource: id => ({
    fhirFetch,
    dispatch,
    store
  }) => {
    const resource = store.selectFhirResourcesById()[id];

    if (!resource) {
      throw Error('NOT FOUND');
    }

    dispatch({
      type: DELETING,
      payload: id
    });
    return fhirFetch("/".concat(resource.resourceType, "/").concat(id), {
      method: 'DELETE'
    }).then(() => {
      store.doRemoveFhirResource(id);
    });
  },
  doSaveFhirResource: resource => ({
    fhirFetch,
    store
  }) => {
    const {
      id,
      resourceType
    } = resource;
    let method = 'post';
    let url = "/".concat(resourceType);

    if (id) {
      url += '/' + id;
      method = 'put';
    }

    return fhirFetch(url, {
      method,
      body: JSON.stringify(resource)
    }).then(res => {
      store.doReceiveFhirResource(res);
      return res;
    });
  },
  doRemoveFhirResource: id => ({
    type: DELETED,
    payload: id
  }),
  doReceiveFhirResource: unknown => ({
    dispatch
  }) => {
    const {
      entry,
      resourceType
    } = unknown;

    if (entry && Array.isArray(entry)) {
      const resources = entry.map(item => item.resource);
      dispatch({
        type: RECEIVED,
        payload: (0, _object.arrayToObject)(resources)
      });
    } else if (resourceType) {
      dispatch({
        type: RECEIVED,
        payload: {
          [unknown.id]: unknown
        }
      });
    } else {
      throw Error('Unknown resource received');
    } // return what was passed in


    return unknown;
  },
  doFhirFetch: (...args) => ({
    fhirFetch
  }) => fhirFetch(...args),
  doFetchFhirResource: fhirReference => ({
    fhirFetch,
    store
  }) => {
    const [resourceType, id] = fhirReference.split('/'); // we have to do this as a search in order to include related resources

    return fhirFetch("/".concat(resourceType, "?_id=").concat(id, "&_include=*")).then(store.doReceiveFhirResource);
  }
};
exports.fhirResourceBundle = fhirResourceBundle;